--[[
  PARTICLE ACCELERATOR POINT DEFENSE
  ==================================
  
  **Version** 1.0
  
  Credits
  -------
  
  *Written by:* The Wicked Flea
  
  *Inspiration & A Tiny Bit of Borrowed Code:* Kyred's Particle Accelerator Cannon
  
  Notes:
  ------
    
    * Kyred, I promise that I only borrowed a LITTLE of your code, and whatever it is
      now is so thoroughly twisted from what you wrote that no one can blame you for it.
  
    * The placable item is still somewhat weak.
    
    * Added an IFF clause.
  
  License:
  --------
  
  Creative Commons NC-BY-SA 3.0 US
  
  http://creativecommons.org/licenses/by-nc-sa/3.0/us/
  
]]--

function percentage(minimum, maximum, value)
  if value <= minimum then
    return 0.0
  elseif value >= maximum then
    return 1.0
  else
    return (value - minimum) / (maximum - minimum)
  end
end

function multiplier(minimum, maximum, current)
  return minimum + (maximum - minimum) * percentage(minimum, maximum, current)
end

function Update(self)
  local targets = { }
  local currentTarget = 1
  
  for actor in MovableMan.Actors do    
    local vector = SceneMan:ShortestDistance(self.Pos, actor.Pos, true)
    
    if actor.Team ~= self.Team and vector.Magnitude < 375 then
      local actorMOIDs = {} -- the following is a table of MOIDs that are safe to shoot at.
      actorMOIDs[actor.RootID] = true -- shoot at our desired target
      for thingamajig in actor.Inventory do
        actorMOIDs[thingamajig.RootID] = true -- and anything it is carrying
      end
      
      local foundMOID = SceneMan:CastMORay(self.Pos, vector, self.RootID, 128, false, 5)
      
      -- strength was determined via logging to console, tweak in _10_'s...
      if SceneMan:CastStrengthSumRay(self.Pos, actor.Pos, 4, 128) < 60
         and (foundMOID == nil or actorMOIDs[foundMOID] == true) then
         
        table.insert(targets, 1, actor)
      end
    end
  end
  
  local targetCount = #targets -- not premature optimization, I assure you.
  local totalShots = 0
  
  for particle in MovableMan.Particles do
    if string.find(particle.PresetName, "PAPD") ~= nil and particle.GoldCost ~= 1337
       and particle.Age < particle.Lifetime * 0.95 then
      
      local distVec = SceneMan:ShortestDistance(particle.Pos, self.Pos, true)
      
      if distVec.Magnitude <= 40 then
        if targetCount > 0 and totalShots < 3*targetCount then -- only 3 shots per actor, per update
          particle.GoldCost = 1337
          particle.HitsMOs  = true
          particle:SetWhichMOToNotHit(self, 1) -- remove for lots of self-kills
          
          local newVel = SceneMan:ShortestDistance(particle.Pos, targets[currentTarget].Pos, true)
          particle.Vel = newVel:SetMagnitude( multiplier(196, 352, newVel.Magnitude) ) -- MAGIC!
          
          if currentTarget + 1 > targetCount then
            currentTarget = 1
          else
            currentTarget = currentTarget + 1
          end
          
          totalShots = totalShots + 1
        else
          -- only "capture" particles that aren't bullets, or fast moving
          -- further particles should want to arc inwards more strongly than those already inside
          particle.Vel = particle.Vel + distVec:SetMagnitude(distVec.Magnitude / 15)
        end
      end
    end
  end
end

function Destroy(self) -- try to avoid this....
  for particle in MovableMan.Particles do
    if string.find(particle.PresetName, "PAPD") ~= nil then
      particle.HitsMOs = true -- the gib particles are the normal ones, and they don't hit MOs by default
    end
  end
end